<?php

declare(strict_types=1);

namespace App\Controller;

use App\Exception\NewsNotFoundException;
use App\Requests\NewsListRequest;
use App\Service\NewsService;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Attribute\Route;

#[Route("/api/v1")]
class NewsController extends AbstractController
{
    public function __construct(private NewsService $newsService) {}

    #[Route('/news', name: 'news', methods: ['GET'])]
    public function news(NewsListRequest $request): Response
    {
        return $this->json(
            $this->newsService->getNewsByRequest($request)
        );
    }

    #[Route('/news/mainNews', name: 'mainNews', methods: ['GET'])]
    public function mainNews(): Response
    {
        return $this->json(
            $this->newsService->getMainNews()
        );
    }

    #[Route('/news/search', name: 'newsSearch', methods: ['GET'])]
    public function newsSearch(): Response
    {
        return $this->json(
            $this->newsService->getNewsSearch()
        );
    }

    #[Route('/news/{newsId}', name: 'newsOne', methods: ['GET'])]
    public function newsOne(Request $request): Response
    {
        try {
            return $this->json(
                $this->newsService->getNewsOneByRequest($request)
            );
        } catch (NewsNotFoundException $e) {
            return $this->json([
                'success' => false,
                'message' => $e->getMessage(),
            ], $e->getCode());
        }
    }
}
