<?php

namespace App\Service;

use App\Entity\News;
use App\Entity\NewsCategory;
use App\Exception\NewsNotFoundException;
use App\Mapper\NewsMapper;
use App\Model\NewsDetailElement;
use App\Model\NewsList;
use App\Model\NewsListingElement;
use App\Repository\Interface\NewsCategoryRepositoryInterface;
use App\Repository\Interface\NewsRepositoryInterface;
use App\Requests\NewsListRequest;
use Ramsey\Collection\Collection;
use Symfony\Component\HttpFoundation\Request;

class NewsService
{
    private const DEFAULT_PAGE = 1;
    private const DEFAULT_LIMIT = 12;

    public function __construct(
        private NewsRepositoryInterface $newsRepository,
        private NewsCategoryRepositoryInterface $newsCategoryRepository
    ) {}

    public function getNewsByRequest(NewsListRequest $request): NewsList
    {
        $page = $request->getRequest()->query->get('page') ?? self::DEFAULT_PAGE;
        $limit = $request->getRequest()->query->get('limit') ?? self::DEFAULT_LIMIT;
        $newsCategory = $request->getRequest()->query->get('news_category');

        return $this->getNews($page, $limit, $newsCategory);
    }

    public function getNews(int $page, int $limit, string|null $newsCategory): NewsList
    {
        $news = new Collection(
            News::class,
            $this->newsRepository->getAll(
                $page,
                $limit,
                $newsCategory
            )
        );
        $newsCategories = new Collection(
            NewsCategory::class,
            $this->newsCategoryRepository->getAll()
        );
        $count = $this->newsRepository->getCount();

        return NewsMapper::mapToNewsList($news, $newsCategories, $page, $limit, $count);
    }

    public function getMainNews(): NewsListingElement
    {
        return NewsMapper::mapToListingElement(
            $this->newsRepository->getMainNews()
        );
    }

    public function getNewsSearch(): NewsDetailElement
    {
        return NewsMapper::mapToDetailElement(
            $this->newsRepository->getMainNews()
        );
    }

    public function getNewsOneByRequest(Request $request): NewsDetailElement
    {
        return $this->getNewsOne(
            $request->get('newsId')
        );
    }

    public function getNewsOne(string $newsId): NewsDetailElement
    {
        $news = $this->newsRepository->getNewsById($newsId);
        if ($news === null) {
            throw new NewsNotFoundException();
        }

        return NewsMapper::mapToDetailElement($news);
    }
}